import socket
import re
import uuid
import platform
import psutil

class System:

    @staticmethod
    def get_host_ip():
        try:
            ip = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
            ip.connect(('8.8.8.8', 80))
            ip_adds = ip.getsockname()[0]
        finally:
            ip.close()
        return ip_adds

    @staticmethod
    def get_mac_address():
        return ":".join(re.findall(r".{2}", uuid.uuid1().hex[-12:]))

    @staticmethod
    def get_platform():
        return platform.system() + " " + platform.machine()

    @staticmethod
    def get_cpu_info():
        return psutil.cpu_percent(interval=1)

    @staticmethod
    def get_memory_info():
        virtual_memory = psutil.virtual_memory()
        return virtual_memory.percent


import logging

class Log:

    logging.basicConfig(
        level=logging.DEBUG,
        format="[%(asctime)s][%(filename)s(line:%(lineno)d)][%(levelname)s]: %(message)s"
    )

    @staticmethod
    def debug(content):
        logging.debug("\033[37m" + str(content) + "\033[0m")

    @staticmethod
    def info(content):
        logging.info("\033[34m" + str(content) + "\033[0m")

    @staticmethod
    def warning(content):
        logging.warning("\033[33m" + str(content) + "\033[0m")

    @staticmethod
    def error(content):
        logging.error("\033[31m" + str(content) + "\033[0m")


import time
import sys
import json
import threading
from .log import Log
from .service import System, Requests, Message, Socket
from .device import Serial, GPIO, Camera
from ..__config__ import __apiUrl__, __imUrl__, __version__

class Init:

    def __init__(self):
        Log.info("Robot " + __version__)

        self.auth_time = time.strftime('%Y%m%d%H%M', time.localtime(time.time()))

        self.version = str(__version__)
        self.app_id = str(sys.argv[1])
        self.app_secret = str(sys.argv[2])
        self.ip = System.get_host_ip()
        self.mac = System.get_mac_address()
        self.platform = System.get_platform()
        self.token = ""

        params = {"app_id": self.app_id, "app_secret": self.app_secret, "robot_mac": self.mac, "robot_ip": self.ip, "robot_platform": self.platform, "robot_version": __version__}

        headers = {
            "Content-Type": "application/json",
            "Robot-Token": sys.argv[1] + "@" + sys.argv[2] + "@" + self.auth_time
        }
        register = Requests.api(__apiUrl__ + "/oauth/robot/register", params, headers, 'GET')
        register_json = register.json()

        if register_json["code"] != 0:
            Log.error("Robot Init Error")
            sys.exit(0)

        self.system_task_thread = threading.Thread(target=self.system_task)
        self.system_task_thread.start()

        self.token = str(register_json["data"]["token"])
        self.name = str(register_json["data"]["robot_title"])
        self.net_ip = str(register_json["data"]["robot_net_ip"])

        self.im = Message(__imUrl__, self.mac, self.app_id, self.app_secret).start()

        self.camera = Camera("/dev/video0")

    def system_task(self):
        while True:
            notice_data = {"message_type": "system_message", "system_message": {"time": time.strftime('%H:%M:%S', time.localtime(time.time())), "cpu": System.get_cpu_info(), "memory": System.get_memory_info()}}
            self.im.publish(json.dumps(notice_data))
            time.sleep(2)


import json
import os
import paho.mqtt.client as mqtt
from queue import Queue
from ..device import Serial

class Message:

    def __init__(self, host, client_id, username, password):
        self.host = host
        self.port = 1883
        self.client_id = client_id
        self.username = username
        self.password = password
        self.mqtt_client = None
        self.timeout = 60
        self.remote_message = Queue(100)

    def on_connect(self, client, userdata, flags, rc):
        self.subscribe("robot/" + self.client_id)

    def on_message(self, client, userdata, msg):
        message_data = json.loads(msg.payload)
        if message_data["message_type"] == "remote_message":
            self.remote_message.put(str(msg.payload, encoding="utf-8"))
        else:
            self.callback_robot_message(str(msg.payload, encoding="utf-8"))

    def read_remote_message(self):
        data = None
        if not self.remote_message.empty():
            data = self.remote_message.get()
        return data

    def subscribe(self, topic):
        self.mqtt_client.subscribe(topic, 0)

    def publish(self, blob):
        self.mqtt_client.publish("robot/" + self.client_id, blob)

    def start(self):
        if self.mqtt_client is None:
            self.mqtt_client = mqtt.Client("robot-" + self.client_id)
            self.mqtt_client.username_pw_set(self.username, self.password)
            self.mqtt_client.on_connect = self.on_connect
            self.mqtt_client.on_message = self.on_message
            self.mqtt_client.connect(self.host, self.port, self.timeout)
            self.mqtt_client.loop_start()
        return self

    def callback_robot_message(self, msg):
        robot_message_json = json.loads(msg)
        if robot_message_json["message_type"] == "robot_run":
            if robot_message_json["robot_run"]["type"] == "update":
                os.system("sudo cp /robot/RoboMentor_Client/robot.py /robot/RoboMentor_Client/robot.bak")
                robot_file = open("/robot/RoboMentor_Client/robot.py", 'w')
                robot_file.write(robot_message_json["robot_run"]["content"])
                robot_file.close()
                notice_data = {"message_type": "robot_run", "robot_run": {"type": "update_success"}}
                self.publish(json.dumps(notice_data))
            if robot_message_json["robot_run"]["type"] == "restart":
                robot_restart = os.popen("sudo sh /robot/RoboMentor_Client/robot_restart.sh " + self.username + " " + self.password).read()
                if robot_restart != "success":
                    notice_data = {"message_type": "robot_run", "robot_run": {"type": "start_error"}}
                    self.publish(json.dumps(notice_data))
        if robot_message_json["message_type"] == "serial_message":
            serial_conn = Serial(robot_message_json["serial_message"]["port"], int(robot_message_json["serial_message"]["rate"]), int(robot_message_json["serial_message"]["size"]))
            serial_conn.write(robot_message_json["serial_message"]["content"].encode('utf-8'))
            if robot_message_json["serial_message"]["switch"]:
                serial_data = serial_conn.read()
                notice_data = {"message_type": "serial_message_read", "serial_message_read": {"content": serial_data}}
                self.publish(json.dumps(notice_data))
        if robot_message_json["message_type"] == "pwm_message":
            serial_conn = Serial(robot_message_json["pwm_message"]["port"], int(robot_message_json["pwm_message"]["rate"]), int(robot_message_json["pwm_message"]["size"]))
            write_data = {"type": robot_message_json["pwm_message"]["type"], "channel": int(robot_message_json["pwm_message"]["channel"]), "width": robot_message_json["pwm_message"]["width"], "rate": int(robot_message_json["pwm_message"]["hz"])}
            serial_conn.write(json.dumps(write_data).encode('utf-8'))

    def stop(self):
        if self.mqtt_client is not None:
            self.mqtt_client.loop_stop()
            self.mqtt_client.disconnect()
            self.mqtt_client = None


import requests
from ..log import Log

class Requests:

    @staticmethod
    def api(url, params, headers, http_method):
        res = ""
        if http_method.upper() == "POST":
            try:
                res = requests.post(url, params, headers=headers)
            except Exception as e:
                Log.error("POST请求出现了异常：{0}".format(e))
        elif http_method.upper() == "GET":
            try:
                res = requests.get(url, params, headers=headers)
            except Exception as e:
                Log.error("GET请求出现了异常：{0}".format(e))
        return res


import os
from ..log import Log

class Camera:

    def __init__(self, device):
        self.status = False

        self.dir = os.path.abspath(os.path.join(os.getcwd(), ".."))

        os.system("sudo " + self.dir + "/micro/micro --device " + device)


import socket

class Socket:

    def __init__(self, port):
        self.socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.socket.bind(("0.0.0.0", port))
        self.socket.listen(10)


import serial
from ..log import Log

class Serial:

    def __init__(self, port, baudrate, size):
        self.status = False

        try:
            self.conn = serial.Serial()
            self.conn.port = port
            self.conn.baudrate = baudrate
            self.conn.bytesize = size
            self.conn.stopbits = serial.STOPBITS_ONE
            self.conn.parity = serial.PARITY_NONE
            self.conn.timeout = 0.2
            self.conn.open()

            self.status = True
        except Exception as e:
            Log.error(str(e))
            return


    def write(self, command):
        if self.status:
            self.conn.write(command)

    def read(self):
        read_data = ""

        if self.status:
            read_data = self.conn.readall()

        return read_data.decode('utf-8')

    def close(self):
        if self.status:
            self.conn.close()